@@ -30,18 +30,24 @@ import glob
 import sys
 import shutil
 import subprocess
-from tempfile import NamedTemporaryFile
+import threading
+import atexit
+import tempfile
 import string
+import stat
+
+from tempfile import NamedTemporaryFile
 from xml.sax.saxutils import escape
 from ConfigParser import ConfigParser
 from gettext import gettext as _
+from urlparse import urlparse
 
 from AptAuth import AptAuth
 import softwareproperties
 import aptsources
 import aptsources.distro
 from aptsources.sourceslist import SourcesList, SourceEntry
-
+from ppa import AddPPASigningKeyThread, expand_ppa_line
 
 
 class SoftwareProperties(object):
@@ -72,11 +78,12 @@ class SoftwareProperties(object):
     # FIXME: some saner way is needed here
     if datadir == None:
       datadir = "/usr/share/software-properties/"
+    self.options = options
     self.datadir = datadir
 
     self.sourceslist = SourcesList()
     self.distro = aptsources.distro.get_distro()
-
+    
     self.seen_server = []
     self.modified_sourceslist = False
 
@@ -92,6 +99,14 @@ class SoftwareProperties(object):
     # apt-key stuff
     self.apt_key = AptAuth()
 
+    atexit.register(self.wait_for_threads)
+
+  def wait_for_threads(self):
+    " wait for all running threads (PPA key fetchers) to exit "
+    for t in threading.enumerate():
+      if t.ident != threading.current_thread().ident:
+        t.join()
+
   def backup_apt_conf(self):
     """Backup all apt configuration options"""
     self.apt_conf_backup = {}
@@ -405,7 +420,7 @@ class SoftwareProperties(object):
   def set_modified_sourceslist(self):
     """The sources list was changed and now needs to be saved and reloaded"""
     self.modified_sourceslist = True
-    if self.options.massive_debug == True:
+    if self.options and self.options.massive_debug:
         self.massive_debug_output()
     self.save_sourceslist()
     self.reload_sourceslist()
@@ -524,6 +539,7 @@ class SoftwareProperties(object):
     """Write the current apt configuration to file"""
     # update the adept file as well if it is there
     conffiles = ["/etc/apt/apt.conf.d/10periodic",
+                 "/etc/apt/apt.conf.d/20auto-upgrades",
                  "/etc/apt/apt.conf.d/15adept-periodic-update"]
 
     # check (beforehand) if one exists, if not create one
@@ -534,6 +550,14 @@ class SoftwareProperties(object):
       print "No config found, creating one"
       open(conffiles[0], "w")
 
+    # ensure /etc/cron.daily/apt is executable
+    ac = "/etc/cron.daily/apt"
+    if os.path.exists(ac):
+      perm = os.stat(ac)[stat.ST_MODE]
+      if not (perm & stat.S_IXUSR):
+        print "file '%s' not executable, fixing" % ac
+        os.chmod(ac, 0755)
+
     # now update them
     for periodic in conffiles:
       # read the old content first
@@ -581,25 +605,43 @@ class SoftwareProperties(object):
     """
     helper that adds the gpg key of the channel to the apt
     keyring *if* the channel is in the whitelist
-    /usr/share/app-install/channels
+    /usr/share/app-install/channels or it is a public Launchpad PPA.
     """
     channel = self._is_line_in_whitelisted_channel(srcline)
     if channel:
       keyp = "%s/%s.key" % (self.CHANNEL_PATH, channel)
-      if os.path.exists(keyp):
-        subprocess.call(["apt-key","add",keyp])
+      self.add_key(keyp)
+    # FIXME: abstract this all alway into the ppa.py file
+    parsed_uri = urlparse(SourceEntry(srcline).uri)
+    if parsed_uri.netloc == 'ppa.launchpad.net':
+      worker = AddPPASigningKeyThread(parsed_uri.path)
+      worker.start()
+
+  def update_interface(self):
+    " abstract interface to keep the UI alive "
 
   def add_source_from_line(self, line):
     """
     Add a source with the given apt line and auto-add
     signing key if we have it in the whitelist
     """
+    (line, file) = expand_ppa_line(line.strip(), self.distro.codename)
     self.check_and_add_key_for_whitelisted_channels(line)
-    self.sourceslist.list.append(SourceEntry(line))
+    new_entry = SourceEntry(line, file)
+    if new_entry.invalid:
+      return False
+    self.sourceslist.list.append(new_entry)
     self.set_modified_sourceslist()
+    return True
 
   def remove_source(self, source):
     """Remove the given source"""
+    # if its a sources.list.d file and it contains only a single line
+    # (the line that we just remove), then all references to that
+    # file are gone and the file is not saved. we work around that
+    # here
+    if source.file != apt_pkg.Config.FindFile("Dir::Etc::sourcelist"):
+      self.sourceslist.list.append(SourceEntry("", file=source.file))
     self.sourceslist.remove(source)
     self.set_modified_sourceslist()
 
@@ -608,11 +650,17 @@ class SoftwareProperties(object):
     if not os.path.exists(path):
         return False
     try:
-        self.apt_key.add(path)
-        return True
+        return self.apt_key.add(path)
     except:
         return False
 
+  def add_key_from_data(self, keydata):
+    "Add a gnupg key from a data string (e.g. copy-n-paste)"
+    tmp = tempfile.NamedTemporaryFile()
+    tmp.write(keydata)
+    tmp.flush()
+    return self.add_key(tmp.name)
+
   def remove_key(self, fingerprint):
     """Remove a gnupg key from the list of trusted software vendors"""
     try:
