@@ -37,8 +37,9 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from PyQt4 import uic
 
-from PyKDE4.kdecore import ki18n
+from PyKDE4.kdecore import ki18n, KUrl
 from PyKDE4.kdeui import KApplication, KMessageBox, KIcon, KWindowSystem
+from PyKDE4.kio import KFileDialog
 
 import softwareproperties
 import softwareproperties.distro
@@ -49,6 +50,12 @@ from DialogEdit import DialogEdit
 from DialogMirror import DialogMirror
 from CdromProgress import CdromProgress
 
+def utf8(str):
+  " helper that takes a string and converts it to utf8 unicode "
+  if isinstance(str, unicode):
+      return str
+  return unicode(str, 'UTF-8')
+
 class SoftwarePropertiesKDEUI(QWidget):
 
     def __init__(self, datadir):
@@ -70,6 +77,13 @@ class SoftwarePropertiesKDE(SoftwareProperties):
 
     self.userinterface = SoftwarePropertiesKDEUI(datadir)
     self.userinterface.setWindowIcon(KIcon("applications-other"))
+    self.userinterface.button_auth_restore.setIcon(KIcon("edit-undo"))
+    self.userinterface.button_add_auth.setIcon(KIcon("list-add"))
+    self.userinterface.button_auth_remove.setIcon(KIcon("list-remove"))
+    self.userinterface.button_remove.setIcon(KIcon("list-remove"))
+    self.userinterface.button_edit.setIcon(KIcon("document-edit"))
+    self.userinterface.button_add.setIcon(KIcon("list-add"))
+    self.userinterface.button_add_cdrom.setIcon(KIcon("media-optical"))
     self.translate_widgets()
     self.userinterface.show()
     if attachWinID is not None:
@@ -107,7 +121,11 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     kapp.connect(self.userinterface.treeview_sources, SIGNAL("itemChanged(QTreeWidgetItem*, int)"), self.on_isv_source_toggled)
     kapp.connect(self.userinterface.treeview_cdroms, SIGNAL("itemChanged(QTreeWidgetItem*, int)"), self.on_cdrom_source_toggled)
     kapp.connect(self.userinterface.treeview_sources, SIGNAL("itemClicked(QTreeWidgetItem*, int)"), self.on_treeview_sources_cursor_changed)
+    kapp.connect(self.userinterface.treeview2, SIGNAL("itemClicked(QTreeWidgetItem*, int)"), self.on_treeview_keys_cursor_changed)
+    button_close = self.userinterface.buttonBox.button(QDialogButtonBox.Close)
+    button_close.setIcon(KIcon("dialog-close"))
     button_revert = self.userinterface.buttonBox.button(QDialogButtonBox.Reset)
+    button_revert.setIcon(KIcon("edit-undo"))
     kapp.connect(button_revert, SIGNAL("clicked()"), self.on_button_revert_clicked)
 
     self.init_distro()
@@ -128,23 +146,22 @@ class SoftwarePropertiesKDE(SoftwareProperties):
       if isinstance(widget, QLabel):
           text = str(widget.text())
           text = _(text)
-          ##FIXME string is a str but in unicode.  PyQt thinks it isn't unicode and any non-ascii characters break, how to fix??
-          text = unicode(text, 'utf-8')
+          text = utf8(text)
           widget.setText(text)
       elif isinstance(widget, QPushButton):
           text = str(widget.text())
           text = _(text)
-          text = unicode(text, 'utf-8')
+          text = utf8(text)
           widget.setText(text)
       elif isinstance(widget, QGroupBox):
           text = str(widget.title())
           text = _(text)
-          text = unicode(text, 'utf-8')
+          text = utf8(text)
           widget.setTitle(text)
       elif isinstance(widget, QCheckBox):
           text = str(widget.text())
           text = _(text)
-          text = unicode(text, 'utf-8')
+          text = utf8(text)
           widget.setText(text)
 
   def init_popcon(self):
@@ -154,7 +171,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if is_helpful != None:
       text = softwareproperties.distro.get_popcon_description(self.distro)
       text = text.replace("\n", "<br />") #silly GTK mixes HTML and normal white space
-      self.userinterface.label_popcon_desc.setText(unicode(text, 'UTF-8'))
+      self.userinterface.label_popcon_desc.setText(utf8(text))
       self.userinterface.checkbutton_popcon.setChecked(is_helpful)
     else:
       self.userinterface.tabWidget.removeTab(4)
@@ -173,17 +190,17 @@ class SoftwarePropertiesKDE(SoftwareProperties):
                                        2 : 7,
                                        3 : 14 }
 
-    self.userinterface.combobox_update_interval.insertItem(99, unicode(_("Daily"), 'utf-8'))
-    self.userinterface.combobox_update_interval.insertItem(99, unicode(_("Every two days"), 'utf-8'))
-    self.userinterface.combobox_update_interval.insertItem(99, unicode(_("Weekly"), 'utf-8'))
-    self.userinterface.combobox_update_interval.insertItem(99, unicode(_("Every two weeks"), 'utf-8'))
+    self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Daily")))
+    self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Every two days")))
+    self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Weekly")))
+    self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Every two weeks")))
 
     update_days = self.get_update_interval()
 
     # If a custom period is defined add a corresponding entry
     if not update_days in self.combobox_interval_mapping.values():
         if update_days > 0:
-            self.userinterface.combobox_update_interval.insertItem(99, unicode(_("Every %s days"), 'utf-8') % update_days)
+            self.userinterface.combobox_update_interval.insertItem(99, utf8(_("Every %s days")) % update_days)
             self.combobox_interval_mapping[4] = update_days
 
     for key in self.combobox_interval_mapping:
@@ -227,10 +244,10 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     # TRANS: %s stands for the distribution name e.g. Debian or Ubuntu
     text = _("%s updates") % self.distro.id
     text = text.replace("Ubuntu", "Kubuntu")
-    self.userinterface.groupBox_updates.setTitle(unicode(text, 'UTF-8'))
+    self.userinterface.groupBox_updates.setTitle(utf8(text))
     # TRANS: %s stands for the distribution name e.g. Debian or Ubuntu
     text = _("%s Software") % self.distro.id
-    text = unicode(text, 'utf-8')
+    text = utf8(text)
     text = text.replace("Ubuntu", "Kubuntu")
     self.userinterface.tabWidget.setTabText(0, text)
 
@@ -250,7 +267,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
         #              first %s is the description of the component
         #              second %s is the code name of the comp, eg main, universe
         label = _("%s (%s)") % (comp.get_description(), comp.name)
-        checkbox = QCheckBox(unicode(label, 'utf-8'), self.userinterface.vbox_dist_comps_frame)
+        checkbox = QCheckBox(utf8(label), self.userinterface.vbox_dist_comps_frame)
         checkbox.setObjectName(comp.name)
         self.checkboxComps[checkbox] = comp
 
@@ -268,7 +285,8 @@ class SoftwarePropertiesKDE(SoftwareProperties):
         self.userinterface.vbox_updates_frame.hide()
     self.checkboxTemplates = {}
     for template in self.distro.source_template.children:
-        checkbox = QCheckBox(unicode(template.description, 'utf-8'), self.userinterface.vbox_updates_frame)
+        checkbox = QCheckBox(utf8(template.description), 
+                             self.userinterface.vbox_updates_frame)
         checkbox.setObjectName(template.name)
         self.checkboxTemplates[checkbox] = template
         # setup the callback and show the checkbutton
@@ -287,7 +305,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     self.mirror_urls = []
     for (name, uri, active) in self.distro.get_server_list():
         ##server_store.append([name, uri, False])
-        self.userinterface.combobox_server.addItem(name)
+        self.userinterface.combobox_server.addItem(utf8(name))
         self.mirror_urls.append(uri)
         if [name, uri] in self.seen_server:
             self.seen_server.remove([name, uri])
@@ -297,11 +315,11 @@ class SoftwarePropertiesKDE(SoftwareProperties):
             self.active_server = self.userinterface.combobox_server.count() - 1
             self.userinterface.combobox_server.setCurrentIndex(self.userinterface.combobox_server.count() - 1)
     for [name, uri] in self.seen_server:
-        self.userinterface.combobox_server.addItem(name)
+        self.userinterface.combobox_server.addItem(utf8(name))
     self.seen_server = seen_server_new
     # add a separator and the option to choose another mirror from the list
     ##FIXME server_store.append(["sep", None, True])
-    self.userinterface.combobox_server.addItem(unicode(_("Other..."), 'utf-8'))
+    self.userinterface.combobox_server.addItem(utf8(_("Other...")))
     self.mirror_urls.append("")
 
     kapp.connect(self.userinterface.combobox_server, SIGNAL("currentIndexChanged(int)"), self.on_combobox_server_changed)
@@ -405,8 +423,8 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if combobox.currentIndex() == self.active_server:
         return
     uri = self.mirror_urls[combobox.currentIndex()]
-    name = unicode(combobox.currentText())
-    if name == unicode(_("Other..."), 'utf-8'):
+    name = utf8(combobox.currentText())
+    if name == utf8(_("Other...")):
         dialogue = DialogMirror(self.userinterface, self.datadir, self.distro, self.custom_mirrors)
         self.translate_widget_children(dialogue)
         res = dialogue.run()
@@ -512,6 +530,14 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     """Setup the user interface parts needed for the key handling"""
     self.userinterface.treeview2.setColumnCount(1)
 
+  def on_treeview_keys_cursor_changed(self, item, column):
+    """set the sensitiveness of the edit and remove button
+       corresponding to the selected channel"""
+    # allow to remove the selected channel
+    self.userinterface.button_auth_remove.setEnabled(True)
+    # disable editing of cdrom sources
+    index = self.userinterface.treeview2.indexOfTopLevelItem(item)
+
   #FIXME revert automation settings too
   def on_button_revert_clicked(self):
     """Restore the source list from the startup of the dialog"""
@@ -544,7 +570,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
         contents = self.render_source(source)
         contents = contents.replace("<b>", "")
         contents = contents.replace("</b>", "")
-        item = QTreeWidgetItem([contents])
+        item = QTreeWidgetItem([utf8(contents)])
         if not source.disabled:
             item.setCheckState(0, Qt.Checked)
         else:
@@ -577,7 +603,10 @@ class SoftwarePropertiesKDE(SoftwareProperties):
   def show_keys(self):
     self.userinterface.treeview2.clear()
     for key in self.apt_key.list():
-      self.userinterface.treeview2.addTopLevelItem(QTreeWidgetItem([key]))
+      self.userinterface.treeview2.addTopLevelItem(QTreeWidgetItem([utf8(key)]))
+
+    if self.userinterface.treeview_sources.topLevelItemCount() < 1:
+      self.userinterface.button_auth_remove.setEnabled(False)
 
   def on_combobox_update_interval_changed(self, index):
     #FIXME! move to backend
@@ -622,8 +651,8 @@ class SoftwarePropertiesKDE(SoftwareProperties):
         example = "deb http://ftp.debian.org sarge main"
     # L10N: the example is of the format: deb http://ftp.debian.org sarge main
     text = _('<p><b>Enter the complete APT line of the<br /> repository that you want to add as source</b></p><p>The APT line includes the type, location and components of a repository,<br /> for example <i>"%s"</i>.</p>' % example)
-    (line, valid) = QInputDialog.getText(self.userinterface, unicode(_("Add APT repository"), 'utf-8'), text)
-    uline = unicode(line)
+    (line, valid) = QInputDialog.getText(self.userinterface, utf8(_("Add APT repository")), text)
+    uline = utf8(line)
     if valid:
         self.add_source_from_line(uline)
         self.show_isv_sources()
@@ -659,15 +688,19 @@ class SoftwarePropertiesKDE(SoftwareProperties):
   def add_key_clicked(self):
     """Provide a file chooser that allows to add the gnupg of a trusted
        software vendor"""
-    filename = QFileDialog.getOpenFileName(self.userinterface, unicode(_("Import key"), 'utf-8'))
-    if filename is not None:
+    home = QDir.homePath()
+    if "SUDO_USER" in os.environ:
+        home = os.path.expanduser("~%s" % os.environ["SUDO_USER"])
+    url = KUrl.fromPath(home)
+    filename = KFileDialog.getOpenFileName(url, '*.asc *.pkr *.skr *.gpg *.gpgkey| PGP keys', self.userinterface, utf8(_("Import key")))
+    if filename:
       if not self.add_key(filename):
         title = _("Error importing selected file")
         text = _("The selected file may not be a GPG key file " \
                 "or it might be corrupt.")
         KMessageBox.sorry(self.userinterface,
-              unicode(text, 'utf-8'),
-              unicode(title, 'utf-8'))
+              utf8(text),
+              utf8(title))
       self.show_keys()
 
   def remove_key_clicked(self):
@@ -676,11 +709,11 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if item == None:
         return
     key = item.text(0)
-    if self.remove_key(key[:8]):
+    if not self.remove_key(key[:8]):
       title = _("Error removing the key")
       text = _("The key you selected could not be removed. "
                "Please report this as a bug.")
-      KMessageBox.sorry(self.userinterface, unicode(text, 'utf-8'), unicode(title, 'utf-8'))
+      KMessageBox.sorry(self.userinterface, utf8(text), utf8(title))
     self.show_keys()
 
   def on_restore_clicked(self):
@@ -694,7 +727,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
     if self.modified_sourceslist == True and self.options.no_update == False:
         messageBox = QMessageBox(self.userinterface)
         messageBox.setIcon(QMessageBox.Information)
-        reloadButton = messageBox.addButton(unicode(_("Reload"), 'utf-8'), QMessageBox.AcceptRole)
+        reloadButton = messageBox.addButton(utf8(_("Reload")), QMessageBox.AcceptRole)
         messageBox.addButton(QMessageBox.Close)
         text = _("<b><big>The information about available software is out-of-date</big></b>\n"
                "\n"
@@ -703,7 +736,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
                "\n"
                "You need a working internet connection to continue.")
         text = text.replace("\n", "<br />")
-        messageBox.setText(unicode(text, 'utf-8'))
+        messageBox.setText(utf8(text))
         messageBox.exec_()
         if (messageBox.clickedButton() == reloadButton):
                 cmd = ["/usr/bin/install-package", "--update"]
@@ -728,7 +761,7 @@ class SoftwarePropertiesKDE(SoftwareProperties):
       progress.close()
       title = _("CD Error")
       text = _("<big><b>Error scanning the CD</b></big>\n\n%s")%msg
-      KMessageBox.sorry(self.userinterface, unicode(text, 'utf-8'), unicode(title, 'utf-8'))
+      KMessageBox.sorry(self.userinterface, utf8(text), utf8(title))
       return
     apt_pkg.Config.Set("Dir::Etc::sourcelist",saved_entry)
     if res == False:
